package com.xynet.marketing.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;

/**
 * 签名加密(超级机器人对接)
 * @author hep
 *
 */
public class RetailSignUtils {

    private static final Logger log = LoggerFactory.getLogger(RetailSignUtils.class);

    private static final String ALGORITHM = "HMAC-SHA1";

    public static String getSignAuth(String host, String uri, String contentType, String methodType, 
    		long timeMillis, String ak, String sk, String body, String queryString) {
        log.info("host={}, uri={}, contentType={}, methodType={}, timeMillis={}, ak={}, sk={}, body={}, queryString={}", 
        		host, uri, contentType, methodType, timeMillis, ak, sk, body, queryString);
        String signedHeaders = getSignedHeaders(host, contentType, timeMillis);
        log.info("signedHeaders = {}", signedHeaders);
        
        String payloadHex = getMD5Hex(body);
        log.info("payloadHex = {}", payloadHex);
        
        String request = getRequest(methodType, uri, queryString, signedHeaders, payloadHex);
        log.info("request = {}", request);
        
        String requextHex = getSHA1Hex(request);
        log.info("requextHex = {}", requextHex);
        
        String stringToSign = getStringToSign(ALGORITHM, requextHex);
        log.info("stringToSign = {}", stringToSign);
        
        String signature = getSignature(stringToSign, sk);
        log.info("signature = {}", signature);
        
        String auth = MD5Encryption.encrypByHMACSHA1(stringToSign, sk);
        log.info("auth = {}", auth);
        return String.format("%s %s:%s", ALGORITHM, ak, signature);
    }


    private static String getSignedHeaders(String host, String contentType, long time) {
        return host.toLowerCase() + ";" + contentType.toLowerCase() + ";" + String.valueOf(time).toLowerCase();
    }

    private static String getMD5Hex(String payload) {
        String payloadUtf8 = new String(payload.getBytes(), StandardCharsets.UTF_8);
        return MD5Encryption.encrypByMD5(payloadUtf8).toLowerCase();
    }

    private static String getSHA1Hex(String request) {
        String requestUtf8 = new String(request.getBytes(), StandardCharsets.UTF_8);
        return MD5Encryption.encrypBySHA1(requestUtf8).toLowerCase();
    }

    private static String getRequest(String httpRequestMethod, String uri, String queryString, String signHeaders, String hex) {
        return httpRequestMethod + "\n" + uri + "\n" + queryString + "\n" + signHeaders + "\n" + hex;
    }

    private static String getStringToSign(String algorithm, String requextHex) {
        return algorithm + "\n" + requextHex;
    }

    private static String getSignature(String stringToSign, String sk) {
        return MD5Encryption.encrypByHMACSHA1(stringToSign, sk);
    }


}
