package com.xynet.marketing.utils.collections.map;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xynet.marketing.utils.Emptys;

import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;

/**
 * HashMap
 */
public class JHashMap<K, V> extends HashMap<K, V> implements JMap<K, V>, Serializable {

    public JHashMap() {

    }

    public JHashMap(int initialCapacity) {
        super(initialCapacity);
    }

    public JHashMap(Map<K, V> map) {
        super(map);
    }

    @Override
    public JMap<K, V> set(K key, V value) {
        this.put(key, value);
        return this;
    }

    @Override
    public JMap<K, V> set(boolean condition, K key, V value) {
        if (condition) {
            this.put(key, value);
        }
        return this;
    }


    @Override
    public V setIfAbsent(K key, V value) {
        return this.putIfAbsent(key, value);
    }

    @Override
    public JMap<K, V> setAll(Map<K, V> map) {
        this.putAll(map);
        return this;
    }

    @Override
    public JMap<K, V> setAll(boolean condition, Map<K, V> map) {
        if (condition) {
            this.putAll(map);
        }
        return this;
    }

    @Override
    public V del(K key) {
        return this.remove(key);
    }

    @Override
    public String getString(K key) {
        return this.get(key) == null ? null : this.get(key).toString();
    }


    @Override
    public Integer getInt(K key) {
        return this.get(key) == null ? null : Integer.parseInt(this.get(key).toString());
    }


    @Override
    public Long getLong(K key) {
        return this.get(key) == null ? null : Long.parseLong(this.get(key).toString());
    }

    @Override
    public Double getDouble(K key) {
        return this.get(key) == null ? null : Double.parseDouble(this.get(key).toString());
    }


    @Override
    public BigDecimal getBigDecimal(K key) {
        return this.get(key) == null ? null : new BigDecimal(this.get(key).toString());
    }


    @Override
    public Boolean getBoolean(K key) {
        return this.get(key) == null ? null : Boolean.parseBoolean(this.get(key).toString());
    }

    @Override
    public Boolean getBoolean(K key, boolean defaultValue) {
        Boolean aBoolean = getBoolean(key);
        return aBoolean == null ? defaultValue : aBoolean;
    }


    @Override
    public LocalDateTime getLocalDateTime(K key) {
        return this.get(key) == null ? null : (LocalDateTime) this.get(key);
    }


    @Override
    public Timestamp getTimestamp(K key) {
        return this.get(key) == null ? null : (Timestamp) this.get(key);
    }


    @Override
    public V get(K key, Class<V> t) {
        V v = this.get(key);
        return v == null ? null : v;
    }

    @Override
    public JMap<String, Object> getMap(K key) {
        Object value = this.get(key);
        if (value == null) {
            return null;
        }
        return new JHashMap<>(BeanUtil.beanToMap(value));
    }

    @Override
    public List<JMap<String, Object>> getListMap(K key) {
        Object value = this.get(key);
        if (value == null) {
            return null;
        }
        List<Object> list = (List<Object>) value;
        List<JMap<String, Object>> result = new ArrayList<>();
        for (Object valueInfo : list) {
            JMap<String, Object> jMap = new JHashMap<>(BeanUtil.beanToMap(valueInfo));
            result.add(jMap);
        }
        return result;
    }

    @Override
    public List<V> getList(K key) {
        V value = this.get(key);
        if (value == null) {
            return null;
        }
        return (List<V>) value;
    }

    /**
     * 将对象List转为MapList
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<Map<String, Object>> copy(List<T> list) {
        if (Emptys.check(list)) {
            List<Map<String, Object>> result = new ArrayList<>(list.size());
            list.forEach(item -> {
                Map<String, Object> stringObjectMap = BeanUtil.beanToMap(item);
                result.add(new JHashMap<>(stringObjectMap));
            });
            return result;
        }
        return new ArrayList<>();
    }

    /**
     * 将对象转为Map
     *
     * @param obj
     * @param <T>
     * @return
     */
    public static <T> Map<String, Object> copy(T obj) {
        if (Emptys.check(obj)) {
            List<Map<String, Object>> copy = copy(Arrays.asList(obj));
            return copy.get(0);
        }
        return null;
    }

    /**
     * 将分页对象转为Map分页对象
     * @param page
     * @return
     * @param <T>
     */
    public static <T> Page<Map<String, Object>> copy(Page<T> page) {
        Page<Map<String, Object>> page2 = new Page();
        page2.setPages(page.getPages()).setCurrent(page.getCurrent()).setSize(page.getSize()).setTotal(page.getTotal());
        if (page.getRecords().size() > 0) {
            List<Map<String, Object>> copy = copy(page.getRecords());
            page2.setRecords(copy);
        }
        return page2;
    }


}
